/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include "stdafx.h"
#include "DataStructures.h"


#ifndef __IMAGETOWINDOWCONVERTER_H__
#define __IMAGETOWINDOWCONVERTER_H__

/**
Converts between image coordinates and client coordinates, and vice versa.  

Assumes that an image of a certain size is displayed in a certain rectangle in the client area.
A window number is associated with the image.  Images of equal size may be displayed above and
below the image with no spacing between adjacent images.  Window numbers increase for images 
further down the display.

Example client area:

                     ...
             +--------------------+
             | image for window 5 |
             +--------------------+
             | image for window 6 |
             +--------------------+
             | image for window 7 |
             +--------------------+
                     ...

Although the coordinate conversion assumes this layout for the display of multiple images, 
it is not necessary that the images actually be visible to the user.  Thus, the conversion 
works no matter how many images are actually displayed on the screen.

The stack of images extends forever in the vertical direction.  Coordinates that are out
of bounds horizontally are clipped to remain inside the vertical strip of images.

Note:  The term 'window' refers to the index within the minirhizotron tube, not the GUI
element on the screen.
*/

class ImageClientConverter
{
public:
  ImageClientConverter();
  ~ImageClientConverter();

  //{
  /// Set image dimensions, the window number of the image, and the rectangle in which 
  ///   the image is displayed in the client area.
  /// All three of these functions must be called at least once to set up the internal 
  ///   data structures, before calling any of the conversion routines.
  void SetClientRect(const CRect& rect);
  void SetImageSize(const CSize& image_size);
  void SetWindow(int window);
  //}

  CRect GetClientRect(int window) const;
  CSize GetImageSize() const;
  int GetWindow() const;

  // returns extended client rect formed by the union of two windows.
  // (This is used to get the client rect in zoomed out mode)
  CRect GetExtendedClientRect(int window1, int window2) const;

  // Convert from client coordinates to image coordinates,
  // automatically computing window in which 'point' lies
  RPoint ClientToImage(const CPoint& point) const;

  // Convert from client coordinates to image coordinates,
  // returning coordinates with respect to specified window
  RPoint ClientToImage(const CPoint& point, int window) const;
  //}

  //{
  /// Convert from image coordinates to client coordinates
  CPoint ImageToClient(const CPoint& point, int window) const;
  CPoint ImageToClient(const RPoint& point) const;
  //}

private:
  CRect m_client_rect;
  CSize m_image_size;
  int m_window;
};




#endif __IMAGETOWINDOWCONVERTER_H__
